@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2016-2022 European Commission (JRC);
+# Copyright 2016-2025 European Commission (JRC);
 # Licensed under the EUPL (the 'Licence');
 # You may not use this work except in compliance with the Licence.
 # You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
@@ -14,7 +14,7 @@ Modules:
 
 .. autosummary::
     :nosignatures:
-    :toctree: _build/formulas
+    :toctree: toctree/formulas
 
     ~parser
     ~builder
