@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2016-2022 European Commission (JRC);
+# Copyright 2016-2025 European Commission (JRC);
 # Licensed under the EUPL (the 'Licence');
 # You may not use this work except in compliance with the Licence.
 # You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
@@ -74,12 +74,18 @@ def _unblock(thisnode, blocked, no_circuit):
             no_circuit[node].clear()
 
 
-def simple_cycles(graph, copy=True):
+def simple_cycles(graph, copy=True, skip_nodes=()):
     # Yield every elementary cycle in python graph G exactly once
     # Expects a dictionary mapping from vertices to iterables of vertices
 
-    if copy:
-        graph = {v: set(nbrs) for v, nbrs in graph.items()}
+    if copy or skip_nodes:
+        skip_nodes = set(skip_nodes)
+        graph = {
+            v: set(nbrs) - skip_nodes
+            for v, nbrs in graph.items()
+            if v not in skip_nodes
+        }
+
     sccs = _strongly_connected_components(graph)
     while sccs:
         scc = sccs.pop()
