@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2016-2022 European Commission (JRC);
+# Copyright 2016-2025 European Commission (JRC);
 # Licensed under the EUPL (the 'Licence');
 # You may not use this work except in compliance with the Licence.
 # You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
@@ -44,14 +44,26 @@ from formulas.errors import (
 from formulas.tokens.operand import Error, XlError
 
 COMPILING = sh.Token('Run')
+DSP = sh.Token('Dsp')
+
+
+def get_shape(r=1, c=1):
+    r = None if r == 1 else r
+    c = None if c == 1 else c
+    return r, c
 
 
 def _init_reshape(base_shape, value):
-    res, (r, c) = np.empty(base_shape, object), value.shape
+    res = np.empty(base_shape, object)
     res[:, :] = getattr(value, '_default', Error.errors['#N/A'])
-    r = None if r == 1 else r
-    c = None if c == 1 else c
-    return res, r, c
+    r, c = get_shape(*value.shape)
+    if r is None and c is not None and base_shape[0] != 1:
+        value = np.repeat(value[:, :base_shape[1]], base_shape[0], axis=0)
+        c = min(base_shape[1], c)
+    elif c is None and r is not None and base_shape[1] != 1:
+        value = np.repeat(value[:base_shape[0]], base_shape[1], axis=1)
+        r = min(base_shape[0], r)
+    return res, r, c, value
 
 
 class Array(np.ndarray):
@@ -64,9 +76,9 @@ class Array(np.ndarray):
             # noinspection PyArgumentList
             return super(Array, self).reshape(shape, *shapes, order=order)
         except ValueError:
-            res, r, c = _init_reshape(shape, self)
+            res, r, c, val = _init_reshape(shape, self)
             try:
-                res[:r, :c] = self
+                res[:r, :c] = val
             except ValueError:
                 res[:, :] = self.collapse(shape)
             return res
@@ -75,14 +87,15 @@ class Array(np.ndarray):
         if self._collapse_value is not None and \
                 tuple(shape) == (1, 1) != self.shape:
             return self._collapse_value
+
         return np.resize(self, shape)
 
     def __reduce__(self):
         reduce = super(Array, self).__reduce__()  # Get the parent's __reduce__.
-        state = dict(
-            _collapse_value=self._collapse_value,
-            _default=self._default
-        ),  # Additional state params to pass to __setstate__.
+        state = {
+            '_collapse_value': self._collapse_value,
+            '_default': self._default
+        },  # Additional state params to pass to __setstate__.
         return reduce[0], reduce[1], reduce[2] + state
 
     def __setstate__(self, state, *args, **kwargs):
@@ -108,10 +121,11 @@ def not_implemented(*args, **kwargs):
 
 def replace_empty(x, empty=0):
     if isinstance(x, np.ndarray):
-        y = x.ravel().tolist()
-        if sh.EMPTY in y:
-            y = [empty if v is sh.EMPTY else v for v in y]
-            return np.asarray(y, object).reshape(*x.shape)
+        obj = np.array(sh.EMPTY, dtype=object)
+        if obj in x:
+            x = np.where(obj == x, empty, x).view(x.__class__)
+    elif x is sh.EMPTY:
+        return empty
     return x
 
 
@@ -176,7 +190,7 @@ FUNCTIONS['ARRAYROW'] = lambda *args: np.asarray(args, object).view(Array)
 
 def get_error(*vals):
     # noinspection PyTypeChecker
-    for v in flatten(vals, None):
+    for v in flatten(vals, None, True):
         if isinstance(v, XlError):
             return v
 
@@ -188,11 +202,51 @@ def raise_errors(*args):
         raise FoundError(err=v)
 
 
-def is_number(number, xl_return=True):
+def _to_number(number):
+    if isinstance(number, (bool, np.bool_)) and number:
+        return np.nan
+    try:
+        return float(number)
+    except (ValueError, TypeError):
+        return np.nan
+
+
+@functools.lru_cache(None)
+def _compile_func(func):
+    return np.frompyfunc(func, 1, 1)
+
+
+def to_number(*args, **kwargs):
+    return _compile_func(_to_number)(*args, **kwargs)
+
+
+def clean_values(values):
+    return values[values != np.array(sh.EMPTY, dtype=object)]
+
+
+def str2complex(string):
+    return complex(string.replace(" ", "").lower().replace("i", "j"))
+
+
+def is_complex(value):
+    if isinstance(value, complex):
+        return True
+    if isinstance(value, str):
+        try:
+            str2complex(value)
+            return True
+        except ValueError:
+            pass
+    return False
+
+
+def is_number(number, xl_return=True, bool_return=False):
     if isinstance(number, (bool, np.bool_)):
-        return False
+        return bool_return
     elif isinstance(number, XlError):
         return xl_return
+    elif number is sh.EMPTY:
+        return False
     else:
         try:
             float(number)
@@ -201,7 +255,6 @@ def is_number(number, xl_return=True):
     return True
 
 
-@functools.lru_cache()
 def _text2num(value):
     if isinstance(value, Array) and not value.shape:
         value = value.tolist()
@@ -263,9 +316,9 @@ def _get_single_args(*args):
 _re_condition = re.compile('(?<!~)[?*]')
 
 
-def _xfilter(accumulator, test_range, condition, operating_range):
+def __xfilter(test_range, condition):
     from .operators import LOGIC_OPERATORS
-    operator, operating_range = '=', np.asarray(operating_range)
+    operator = '='
     if isinstance(condition, str):
         for k in LOGIC_OPERATORS:
             if condition.startswith(k) and condition != k:
@@ -280,11 +333,7 @@ def _xfilter(accumulator, test_range, condition, operating_range):
                     tuple(map(lambda v: '.%s' % v, it)) + ('',)
                 ), ()))).match
                 f = lambda v: isinstance(v, str) and bool(match(v))
-                b = np.vectorize(f, otypes=[bool])(test_range['raw'])
-                try:
-                    return accumulator(operating_range[b])
-                except FoundError as ex:
-                    return ex.err
+                return np.vectorize(f, otypes=[bool])(test_range['raw'])
             elif any(v in condition for v in ('~?', '~*')):
                 condition = condition.replace('~?', '?').replace('~*', '*')
         from ..tokens.operand import Number, Error
@@ -301,7 +350,12 @@ def _xfilter(accumulator, test_range, condition, operating_range):
         condition = _text2num(condition)
 
     from .operators import _get_type_id
-    type_id, operator = _get_type_id(condition), LOGIC_OPERATORS[operator]
+    type_id = _get_type_id(condition)
+    if operator == '=' and type_id == 1:
+        condition = condition.casefold()
+        operator = lambda x, y: x.casefold() == y
+    else:
+        operator = LOGIC_OPERATORS[operator]
 
     @functools.lru_cache()
     def check(value):
@@ -313,42 +367,66 @@ def _xfilter(accumulator, test_range, condition, operating_range):
         b = np.vectorize(check, otypes=[bool])(test_range['num'])
     else:
         b = np.vectorize(check, otypes=[bool])(test_range['raw'])
+    return b
+
+
+def _xfilter(accumulator, operating_range, test_ranges, *conditions):
+    operating_range = np.asarray(operating_range)
     try:
+        b = None
+        for test_range, condition in zip(test_ranges, conditions):
+            if b is None:
+                b = __xfilter(test_range, condition)
+            else:
+                b &= __xfilter(test_range, condition)
         return accumulator(operating_range[b])
     except FoundError as ex:
         return ex.err
 
 
-_xfilter = np.vectorize(_xfilter, otypes=[object], excluded={0, 1, 3})
+_xfilter = np.vectorize(_xfilter, otypes=[object], excluded={0, 1, 2})
 
 
 def xfilter(accumulator, test_range, condition, operating_range=None):
     operating_range = test_range if operating_range is None else operating_range
     # noinspection PyTypeChecker
     test_range = {'raw': replace_empty(test_range, '')}
-    res = _xfilter(accumulator, test_range, condition, operating_range)
+    res = _xfilter(accumulator, operating_range, [test_range], condition)
     return res.view(Array)
 
 
-def flatten(v, check=is_number):
+def xfilters(accumulator, operating_range, test_range, condition, *args):
+    # sanity-check: the rest has to be even-length
+    if len(args) % 2:
+        raise ValueError(
+            "Additional arguments must be supplied in (test_range, condition) pairs."
+        )
+    test_ranges = [test_range, *args[0::2]]
+    operating_range = test_range if operating_range is None else operating_range
+
+    # noinspection PyTypeChecker
+    test_ranges = [{'raw': replace_empty(v, '')} for v in test_ranges]
+    res = _xfilter(
+        accumulator, operating_range, test_ranges, condition, *args[1::2]
+    )
+    return res.view(Array)
+
+
+def flatten(v, check=is_number, drop_empty=False):
     if isinstance(v, np.ndarray):
-        if not check:
+        if drop_empty or check is is_number or check is is_not_empty:
+            v = v[v != np.array(sh.EMPTY, dtype=object)]
+        if not check or check is is_not_empty:
             yield from v.ravel()
         else:
             yield from filter(check, v.ravel())
     elif not isinstance(v, str) and isinstance(v, Iterable):
         for el in v:
-            yield from flatten(el, check)
+            yield from flatten(el, check, drop_empty)
     elif not check or check(v):
         yield v
 
 
-# noinspection PyUnusedLocal
-def value_return(res, *args):
-    res._collapse_value = Error.errors['#VALUE!']
-    return res
-
-
 def convert_nan(value, default=Error.errors['#NUM!']):
     return value if np.isfinite(value) else default
 
@@ -359,6 +437,38 @@ def convert_noshp(value):
     return value
 
 
+def args2vals(args):
+    return (np.ravel(v)[0] for v in args)
+
+
+def args2list(max_shape, shapes, *args):
+    it = []
+    for arg, shape in zip(args, shapes):
+        if not shape or shape[0] == 1:
+            arg = np.tile(arg, max_shape)
+        elif shape[0] != max_shape:
+            raise BroadcastError()
+        it.append(arg)
+    return map(args2vals, zip(*it))
+
+
+_get_first_val = np.vectorize(
+    lambda x: x[0][0] if isinstance(x, list) else x, otypes=[object]
+)
+
+
+def return_2d_func(res, *args):
+    if not res.shape:
+        res = res.item()
+        res = np.asarray(res, object).view(Array)
+    elif res.shape == (1, 1):
+        res = res[0, 0]
+        res = np.asarray(res, object).view(Array)
+    else:
+        res = _get_first_val(res)
+    return res
+
+
 def wrap_ufunc(
         func, input_parser=lambda *a: map(float, a), check_error=get_error,
         args_parser=lambda *a: map(replace_empty, a), otype=Array,
@@ -383,12 +493,27 @@ def wrap_ufunc(
         try:
             args = tuple(args_parser(*args))
             with np.errstate(divide='ignore', invalid='ignore'):
-                res = np.vectorize(safe_eval, **kw)(*args)
+                if len(args) >= 32:
+                    shapes = [np.shape(arg) for arg in args]
+                    max_shape = max((s or (1,))[0] for s in shapes)
+                    if max_shape == 1:
+                        res = np.asarray([[
+                            safe_eval(*args2vals(args))
+                        ]], object).view(otype)
+                    else:
+                        res = np.asarray([safe_eval(*v) for v in args2list(
+                            max_shape, shapes, *args
+                        )], object).view(otype)
+                else:
+                    res = np.vectorize(safe_eval, **kw)(*args)
             try:
                 res = res.view(otype)
             except AttributeError:
                 res = np.asarray([[res]], object).view(otype)
             return return_func(res, *args)
+        except FoundError as ex:
+            res = np.asarray([[ex.err]], object).view(otype)
+            return return_func(res, *args)
         except ValueError as ex:
             try:
                 np.broadcast(*args)
