@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2016-2022 European Commission (JRC);
+# Copyright 2016-2025 European Commission (JRC);
 # Licensed under the EUPL (the 'Licence');
 # You may not use this work except in compliance with the Licence.
 # You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
@@ -13,11 +13,14 @@ import math
 import functools
 import collections
 import numpy as np
+import schedula as sh
 from decimal import Decimal, ROUND_HALF_UP
 from . import (
-    get_error, raise_errors, is_number, flatten, wrap_ufunc, wrap_func,
-    replace_empty, Error, xfilter, wrap_impure_func, COMPILING
+    raise_errors, is_number, flatten, wrap_ufunc, wrap_func, get_error,
+    replace_empty, Error, xfilter, wrap_impure_func, COMPILING, to_number,
+    clean_values, Array, XlError, xfilters
 )
+from ..errors import FoundError
 
 # noinspection PyDictCreation
 FUNCTIONS = {}
@@ -149,7 +152,9 @@ def _factdouble(x):
 
 def xfactdouble(number):
     raise_errors(number)
-    x = list(flatten(replace_empty(number), None))[0]
+    x = next(flatten(number, None))
+    if x is sh.EMPTY:
+        x = 0
     if isinstance(x, bool):
         return Error.errors['#VALUE!']
     with np.errstate(divide='ignore', invalid='ignore'):
@@ -185,7 +190,7 @@ def xgcd(*args):
 
 
 FUNCTIONS['GCD'] = wrap_func(xgcd)
-FUNCTIONS['INT'] = wrap_ufunc(int)
+FUNCTIONS['INT'] = wrap_ufunc(math.floor)
 FUNCTIONS['ISO.CEILING'] = FUNCTIONS['CEILING.PRECISE']
 
 
@@ -201,6 +206,34 @@ FUNCTIONS['LOG'] = wrap_ufunc(
 FUNCTIONS['LN'] = wrap_ufunc(np.log)
 
 
+def xmdeterm(x, func=np.linalg.det):
+    raise_errors(x)
+    if x.shape[0] != x.shape[1]:
+        return Error.errors['#VALUE!']
+    x = np.reshape(
+        tuple(flatten(x, lambda v: not isinstance(v, str) and is_number(v))),
+        x.shape
+    )
+    try:
+        return np.around(func(x.astype(float)), 15).view(Array)
+    except np.linalg.LinAlgError:
+        return Error.errors['#NUM!']
+
+
+FUNCTIONS['MDETERM'] = wrap_func(xmdeterm)
+FUNCTIONS['MINVERSE'] = wrap_func(
+    functools.partial(xmdeterm, func=np.linalg.inv)
+)
+
+
+def xmmult(x, y):
+    raise_errors(x, y)
+    return np.dot(x, y).astype(float).view(Array)
+
+
+FUNCTIONS['MMULT'] = wrap_func(xmmult)
+
+
 def xmod(x, y):
     return y == 0 and Error.errors['#DIV/0!'] or np.mod(x, y)
 
@@ -222,12 +255,66 @@ def xmround(*args):
 FUNCTIONS['MROUND'] = wrap_func(xmround)
 
 
+def xmultinominal(*args):
+    from . import convert2float
+    args = tuple(flatten(map(replace_empty, args), None))
+    if any(isinstance(v, bool) for v in args):
+        raise FoundError(err=Error.errors['#VALUE!'])
+    nums = np.asarray(tuple(convert2float(*args))).astype(int)
+    if (nums < 0).any():
+        raise FoundError(err=Error.errors['#NUM!'])
+    log_num = math.lgamma(nums.sum() + 1.0)
+    log_den = sum(math.lgamma(x + 1.0) for x in nums)
+    return round(np.exp(log_num - log_den), 11)
+
+
+FUNCTIONS['MULTINOMIAL'] = wrap_func(xmultinominal)
+
+
+def return_func(res, inp):
+    shape = np.asarray(inp).shape
+    if len(shape) == 2:
+        return np.asarray([
+            v if isinstance(v, XlError) or v is 1.0 else v[0][0]
+            for v in res.ravel()
+        ], dtype=object).reshape(shape).view(Array)
+    return res
+
+
+def xmunit(x):
+    if x > 1:
+        return np.identity(x)
+    elif x == 1:
+        return 1.0
+    return Error.errors['#VALUE!']
+
+
+FUNCTIONS['_XLFN.MUNIT'] = FUNCTIONS['MUNIT'] = wrap_ufunc(
+    xmunit, input_parser=lambda *a: map(int, a), return_func=return_func,
+    check_nan=False
+)
+
+
 def xodd(x):
     v = math.ceil(abs(x)) // 2 * 2 + 1
     return -v if x < 0 else v
 
 
 FUNCTIONS['ODD'] = wrap_ufunc(xodd)
+
+
+def xpercentof(data_subset, data_all):
+    if get_error(data_subset, data_all):
+        raise FoundError(err=Error.errors['#NUM!'])
+    check = lambda v: not isinstance(v, str) and is_number(v)
+    num = sum(flatten(data_subset, check), 0)
+    den = sum(flatten(data_all, check), 0)
+    if den:
+        return num / den
+    raise FoundError(err=Error.errors['#DIV/0!'])
+
+
+FUNCTIONS['_XLFN.PERCENTOF'] = FUNCTIONS['PERCENTOF'] = wrap_func(xpercentof)
 FUNCTIONS['PI'] = lambda: math.pi
 
 
@@ -258,10 +345,7 @@ def xrandbetween(bottom, top):
     return bottom + dx * np.random.rand()
 
 
-FUNCTIONS['RANDBETWEEN'] = wrap_ufunc(
-    xrandbetween, input_parser=lambda *a: a,
-    check_error=lambda *a: get_error(*a[::-1])
-)
+FUNCTIONS['RANDBETWEEN'] = wrap_ufunc(xrandbetween, input_parser=lambda *a: a)
 
 
 def _xroman(form):
@@ -325,16 +409,9 @@ def xsumproduct(*args):
     # Check all arrays are the same length
     # Excel returns #VAlUE! error if they don't match
     raise_errors(args)
-    assert len(set(arg.size for arg in args)) == 1
-    inputs = []
-    for a in args:
-        a = np.asarray(a).ravel()
-        x = np.zeros_like(a, float)
-        b = np.vectorize(is_number)(a)
-        x[b] = a[b]
-        inputs.append(x)
-
-    return np.sum(np.prod(inputs, axis=0))
+    inp = np.asarray(args).reshape((len(args), -1))
+    inp = inp[:, ~(inp == np.array(sh.EMPTY, dtype=object)).any(axis=0)]
+    return np.sum(np.prod(np.nan_to_num(to_number(inp).astype(float)), axis=0))
 
 
 FUNCTIONS['SUMPRODUCT'] = wrap_func(xsumproduct)
@@ -354,21 +431,62 @@ def xsrqtpi(number):
 FUNCTIONS['SQRTPI'] = wrap_func(xsrqtpi)
 
 
-def xsum(*args, func=sum):
+def xsum(*args, func=np.sum):
     raise_errors(args)
-    inputs = []
+    inp = []
     for a in args:
         if isinstance(a, str) and not is_number(a):
             raise ValueError
         elif isinstance(a, bool):
             a = float(a)
-        inputs.append(a)
-    return func(tuple(map(float, flatten(inputs))))
+        inp.append(np.asarray(a).reshape(1, -1))
+    inp = to_number(clean_values(np.concatenate(inp, 1))).astype(float).ravel()
+    return func(inp[~np.isnan(inp)])
 
 
 FUNCTIONS['PRODUCT'] = wrap_func(functools.partial(xsum, func=np.prod))
 FUNCTIONS['SUM'] = wrap_func(xsum)
 FUNCTIONS['SUMIF'] = wrap_func(functools.partial(xfilter, xsum))
+FUNCTIONS['SUMIFS'] = wrap_func(functools.partial(xfilters, xsum))
+
+FUNCTIONS['SUMSQ'] = wrap_func(functools.partial(
+    xsum, func=lambda v: np.sum(np.square(v))
+))
+
+
+def sumx2my2(array_x, array_y, func=lambda x, y: np.sum(x ** 2 - y ** 2)):
+    raise_errors(array_x, array_y)
+    array_x = np.asarray(array_x).reshape(-1, 1)
+    array_y = np.asarray(array_y).reshape(-1, 1)
+
+    if array_x.size != array_y.size:
+        raise FoundError(err=Error.errors['#N/A'])
+    inp = np.concatenate((array_x, array_y), 1)
+    from .look import _vect_get_type_id
+    inp = replace_empty(inp[(
+            inp != np.array(sh.EMPTY, dtype=object)
+    ).all(axis=1)])
+    inp = inp[(_vect_get_type_id(inp) == 0).all(axis=1)]
+    if inp.size == 0:
+        raise FoundError(err=Error.errors['#DIV/0!'])
+
+    return func(*inp.astype(float).T)
+
+
+FUNCTIONS['SUMX2MY2'] = wrap_func(sumx2my2)
+FUNCTIONS['SUMX2PY2'] = wrap_func(functools.partial(
+    sumx2my2, func=lambda x, y: np.sum(x ** 2 + y ** 2)
+))
+FUNCTIONS['SUMXMY2'] = wrap_func(functools.partial(
+    sumx2my2, func=lambda x, y: np.sum((x - y) ** 2)
+))
+
 FUNCTIONS['TAN'] = wrap_ufunc(np.tan)
 FUNCTIONS['TANH'] = wrap_ufunc(np.tanh)
-FUNCTIONS['TRUNC'] = wrap_ufunc(functools.partial(xround, func=math.trunc))
+
+
+def xtrunc(x, d=0, func=math.trunc):
+    return xround(x, d=d, func=func)
+
+
+FUNCTIONS['TRUNC'] = wrap_ufunc(xtrunc)
