@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2016-2022 European Commission (JRC);
+# Copyright 2016-2025 European Commission (JRC);
 # Licensed under the EUPL (the 'Licence');
 # You may not use this work except in compliance with the Licence.
 # You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
@@ -15,7 +15,7 @@ import regex
 from .errors import TokenError, FormulaError, ParenthesesError
 from .tokens.operand import String, Error, Number, Range
 from .tokens.operator import OperatorToken, Separator, Intersect
-from .tokens.function import Function, Array
+from .tokens.function import Function, Array, Lambda
 from .tokens.parenthesis import Parenthesis
 from .builder import AstBuilder
 
@@ -30,10 +30,13 @@ class Parser:
     )
     ast_builder = AstBuilder
     filters = [
-        Error, String, Number, Range, OperatorToken, Separator, Function, Array,
-        Parenthesis, Intersect
+        Error, String, Number, Lambda, Range, OperatorToken, Separator,
+        Function, Array, Parenthesis, Intersect
     ]
 
+    def __init__(self, is_cell=False):
+        self.is_cell = is_cell
+
     def is_formula(self, value):
         return self.formula_check.match(value) or Error._re.match(value)
 
@@ -49,7 +52,7 @@ class Parser:
         while expr:
             for f in filters:
                 try:
-                    token = f(expr, context)
+                    token = f(expr, context, self)
                     token.ast(tokens, stack, builder)
                     expr = expr[token.end_match:]
                     break
@@ -66,6 +69,6 @@ class Parser:
                 raise ParenthesesError()
             builder.append(stack.pop())
         if len(builder) != 1:
-            FormulaError(expression)
+            raise FormulaError(expression)
         builder.finish()
         return tokens, builder
