@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2016-2022 European Commission (JRC);
+# Copyright 2016-2025 European Commission (JRC);
 # Licensed under the EUPL (the 'Licence');
 # You may not use this work except in compliance with the Licence.
 # You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
@@ -22,12 +22,12 @@ class Operator(Token):
     # http://office.microsoft.com/en-us/excel-help/calculation-operators-and-
     # precedence-HP010078886.aspx
     _precedences = {
-        ':': 8, ' ': 8, ',': 8, 'u-': 7, 'u+': 7, '%': 6, '^': 5, '*': 4,
-        '/': 4, '+': 3, '-': 3, '&': 2, '=': 1, '<': 1, '>': 1, '<=': 1,
+        ':': 8, ' ': 8, ',': 8, 'u-': 7, 'u+': 7, '@': 7, '%': 6, '^': 5,
+        '*': 4, '/': 4, '+': 3, '-': 3, '&': 2, '=': 1, '<': 1, '>': 1, '<=': 1,
         '>=': 1, '<>': 1
     }
     _n_args = collections.defaultdict(lambda: 2)
-    _n_args.update({'u-': 1, 'u+': 1, '%': 1})
+    _n_args.update({'u-': 1, 'u+': 1, '@': 1, '%': 1})
 
     _re_process = None
     _replace = ' '
@@ -38,10 +38,15 @@ class Operator(Token):
     def update_input_tokens(self, *tokens):
         if self.name in ' ,:':
             self.attr['is_ranges'] = True
-            from .operand import Range
+            from .operand import Range, Error
+            from .function import Function
             for t in tokens:
                 if isinstance(t, Range):
                     t.attr['is_ranges'] = True
+                elif isinstance(t, Function) and self.name in ':':
+                    t.attr['is_ranges'] = True
+                elif isinstance(t, Error) and t.name == '#REF!':
+                    t.attr['is_ranges'] = True
                 elif not t.attr.get('is_ranges', False):
                     raise FormulaError()
         else:
@@ -53,6 +58,8 @@ class Operator(Token):
             expr = '{}%'.format(*expr)
         elif name in ('u-', 'u+'):
             expr = '{}{}'.format(name[1], *expr)
+        elif name == '@':
+            expr = '@{}'.format(*expr)
         elif name in ' ,:':
             expr = '(%s)' % ('%s ' % name.strip(' ')).join(expr)
         else:
@@ -63,12 +70,14 @@ class Operator(Token):
     def get_n_args(self):
         return self._n_args[self.name]
 
-    def process(self, match, context=None):
+    def process(self, match, context=None, parser=None):
         if self._re_process:
             s = match.groups()[0].replace(self._replace, '')
             match = self._re_process.match(s)
         if match:
-            return super(Operator, self).process(match, context=context)
+            return super(Operator, self).process(
+                match, context=context, parser=parser
+            )
         return {}
 
     @property
@@ -84,6 +93,14 @@ class Operator(Token):
             if not (b or isinstance(t, Operand)):
                 self.attr['name'] = 'u%s' % self.name
                 _update_n_args(stack)
+        elif self.name == '@':
+            from .operand import Operand
+            t = tokens[max(tokens.index(self) - 1, 0)]
+            b = isinstance(t, Parenthesis) and t.has_end
+            b |= isinstance(t, Operator) and t.name == '%'
+            if not (b or isinstance(t, Operand)):
+                self.attr['name'] = '%s' % self.name
+                _update_n_args(stack)
 
     def ast(self, tokens, stack, builder):
         super(Operator, self).ast(tokens, stack, builder)
@@ -109,9 +126,14 @@ class Separator(Operator):
     _re_process = regex.compile(r'^\s*(?P<name>,)$')
 
     def ast(self, tokens, stack, builder):
-        if tokens and isinstance(tokens[-1], Separator):
-            from .operand import Empty
-            Empty().ast(tokens, stack, builder)
+        if tokens:
+            lt = tokens[-1]
+            from .operand import String
+            if isinstance(lt, Separator) or (
+                    lt.get_name == '(' and not isinstance(lt, String)
+            ):
+                from .operand import Empty
+                Empty().ast(tokens, stack, builder)
         super(Operator, self).ast(tokens, stack, builder)
         while stack and not stack[-1].has_start:
             builder.append(stack.pop())
@@ -121,13 +143,13 @@ class Separator(Operator):
 
 class OperatorToken(Operator):
     _re = regex.compile(
-        r'^(\s*([<>]=|<>|[\*\/\^&<>=])(?=\s*[\+\-])|\s*%+|[\+\-\*\/\^&<>=\s:]+)'
+        r'^(\s*([<>]=|<>|[\*\/\^&<>=])(?=\s*[\+\-])|\s*%+|[\+\-\*\/\^&<>=\s:@]+)'
     )
     _re_process = regex.compile(
-        r'^\s*(?P<name>(?P<sum_minus>[\+\s\-]+)|[<>]?=|<>|[\*\/\^&\%:<>])$'
+        r'^\s*(?P<name>(?P<sum_minus>[\+\s\-]+)|[<>]?=|<>|[\*\/\^&\%:<>@])$'
     )
 
-    def process(self, match, context=None):
+    def process(self, match, context=None, parser=None):
         attr = super(OperatorToken, self).process(match, context=context)
         if 'sum_minus' in attr:
             attr['name'] = '-+'[attr['sum_minus'].count('-') % 2 == 0]
