@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2016-2022 European Commission (JRC);
+# Copyright 2016-2025 European Commission (JRC);
 # Licensed under the EUPL (the 'Licence');
 # You may not use this work except in compliance with the Licence.
 # You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
@@ -24,7 +24,13 @@ class Parenthesis(Token):
     n_args = 0
 
     def ast(self, tokens, stack, builder):
-        from . operand import Operand
+        from .operator import Separator
+        from .operand import Operand
+        if tokens and isinstance(
+                tokens[-1], Separator
+        ) and self.get_name == ')':
+            from .operand import Empty
+            Empty().ast(tokens, stack, builder)
         if self.has_start and tokens and isinstance(tokens[-1], Operand):
             raise TokenError
         super(Parenthesis, self).ast(tokens, stack, builder)
