@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2016-2022 European Commission (JRC);
+# Copyright 2016-2025 European Commission (JRC);
 # Licensed under the EUPL (the 'Licence');
 # You may not use this work except in compliance with the Licence.
 # You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
@@ -20,6 +20,7 @@ from formulas.functions import wrap_ranges_func
 @ddt.ddt
 class TestParser(unittest.TestCase):
     @ddt.data(
+        ('=FIND("(",A1,1)', 'FIND((,A1,1)'),
         ('=IF(A1>=-1,1,0)', 'IF(A1>=u-1,1,0)'),
         ("=SUM('Sheet 1'!A2:'Sheet 1'!A2)", "SUM('SHEET 1'!A2:'SHEET 1'!A2)"),
         ('=SUM(5,5, ,6)', 'SUM(5,5,,6)'),
@@ -63,7 +64,8 @@ class TestParser(unittest.TestCase):
         ('=10  ^  -  2', '10^u-2'),
         ('=10^- + -  + + +2', '10^u+2'),
         ('=ATAN2( 10 , 2)', 'ATAN2(10,2)'),
-        ('=DAYS360( 10 , 2)', 'DAYS360(10,2)')
+        ('=DAYS360( 10 , 2)', 'DAYS360(10,2)'),
+        ('=FIRSTPARAMEMPTY(,,1)', 'FIRSTPARAMEMPTY(,,1)'),
     )
     def test_valid_formula(self, case):
         inputs, result = case
